package com.ym.mec.thirdparty.message;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Component;

import com.ym.mec.thirdparty.exception.ThirdpartyException;
import com.ym.mec.thirdparty.message.vendors.JiguangPushPlugin;
import com.ym.mec.thirdparty.message.vendors.MOxintongSMSPlugin;
import com.ym.mec.thirdparty.message.vendors.ShiyuanSMSPlugin;

@Component
public class MessageSenderPluginContext implements ApplicationContextAware {

	public enum MessageSender {

		JIGUANG, MOXINGTONG, SHIYUAN;
	}

	private MessageSenderPlugin messageSenderPlugin;

	private ApplicationContext applicationContext;

	private final Map<String, String> mapper = new HashMap<String, String>() {

		/**
		 * 
		 */
		private static final long serialVersionUID = -3964872523891264522L;

		{
			put("jiguang", StringUtils.uncapitalize(JiguangPushPlugin.class.getSimpleName()));
			put("moxingtong", StringUtils.uncapitalize(MOxintongSMSPlugin.class.getSimpleName()));
			put("shiyuan", StringUtils.uncapitalize(ShiyuanSMSPlugin.class.getSimpleName()));
		}
	};

	/**
	 * 发送消息至目的地
	 * @param messageSender 消息发送方
	 * @param subject 消息主题
	 * @param content 消息内容
	 * @param receiver 收件人
	 * @param url 链接地址
	 * @return 是否发送成功
	 * @throws IOException
	 */
	public boolean send(MessageSender messageSender, String subject, String content, String receiver, String url) throws IOException {

		messageSenderPlugin = getMessageSenderPlugin(messageSender);

		return messageSenderPlugin.send(subject, content, receiver, url);
	}

	/**
	 * 批量发送消息至目的地
	 * @param messageSender 消息发送方
	 * @param subject 消息主题
	 * @param content 消息内容
	 * @param receivers 收件人列表
	 * @param url 链接地址
	 * @return 是否发送成功
	 * @throws IOException
	 */
	public boolean batchSend(MessageSender messageSender, String subject, String content, String[] receivers, String url) throws IOException {

		messageSenderPlugin = getMessageSenderPlugin(messageSender);

		return messageSenderPlugin.batchSend(subject, content, receivers, url);
	}

	@Override
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		this.applicationContext = applicationContext;
	}

	private MessageSenderPlugin getMessageSenderPlugin(MessageSender messageSender) {
		String beanId = mapper.get(StringUtils.lowerCase(messageSender.name()));

		if (StringUtils.isBlank(beanId)) {
			throw new ThirdpartyException("消息发送方不存在");
		}

		return applicationContext.getBean(beanId, MessageSenderPlugin.class);
	}

}
