package com.ym.mec.web.service;

import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.BaseService;
import com.ym.mec.web.dal.dto.HighClassGroup;
import com.ym.mec.web.dal.dto.TeacherMusicClassInfoDto;
import com.ym.mec.web.dal.dto.TeacherVipClassInfoDto;
import com.ym.mec.web.dal.entity.ClassGroup;
import com.ym.mec.web.dal.entity.ClassGroupRelation;
import com.ym.mec.web.dal.page.TeacherCourseQueryInfo;
import com.ym.mec.web.dal.page.VipClassQueryInfo;
import org.apache.ibatis.annotations.Param;
import org.springframework.data.domain.Page;

import java.util.List;

public interface ClassGroupService extends BaseService<Integer, ClassGroup> {

    /**
     * 查询老师乐团课课程
     *
     * @param teacherId
     * @return
     */
    List<ClassGroup> findClassGroup4Teacher(Integer teacherId);

    /**
     * 合奏班相关班级获取
     *
     * @param mixClassGroupId
     * @return
     */
    List<ClassGroup> findClassGroup(Integer musicGroupId, Integer mixClassGroupId);

    /**
     * 根据乐团编号查询单技班列表
     *
     * @param musicGroupId
     * @return
     */
    List<ClassGroup> findAllNormalClassGroupByMusicGroupId(int musicGroupId);


    /**
     * 根据乐团编号查询合奏班列表
     *
     * @param musicGroupId
     * @return
     */
    List<ClassGroup> findAllMixClassGroupByMusicGroupId(int musicGroupId);

    /**
     * 添加单技班
     *
     * @param classGroup
     * @return
     */
    ClassGroup addClassGroup(ClassGroup classGroup) throws Exception;

    /**
     * 添加合奏班
     *
     * @param classGroupIds
     * @return
     * @throws Exception
     */
    ClassGroup addMixClassGroup(int musicGroupId, String name, String classGroupIds) throws Exception;

    /**
     * 跟班级ids查询班级信息
     *
     * @param ids
     * @return
     */
    List<ClassGroup> findClassGroupByIds(String ids);


    /**
     * 提高班批量添加
     * @param highClassGroupList
     * @return
     * @throws Exception
     */
    List<HighClassGroup> addHighClassGroup(List<HighClassGroup> highClassGroupList) throws Exception;

    /**
     * 获取老师乐团课程信息列表
     * @param teacherId
     * @return
     */
    List<TeacherMusicClassInfoDto> getTeacherMusicClass(Integer teacherId);

    /**
     * 获取老师VIP课程信息列表
     * @param queryInfo
     * @return
     */
    PageInfo<TeacherVipClassInfoDto> getTeacherVipClass(VipClassQueryInfo queryInfo);
}