package com.ym.mec.web.service.impl;

import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.web.dal.dao.ClassGroupDao;
import com.ym.mec.web.dal.dao.TeacherDao;
import com.ym.mec.web.dal.dto.HighClassGroup;
import com.ym.mec.web.dal.dto.TeacherMusicClassInfoDto;
import com.ym.mec.web.dal.dto.TeacherVipClassInfoDto;
import com.ym.mec.web.dal.entity.*;
import com.ym.mec.web.dal.enums.ClassGroupStudentStatusEnum;
import com.ym.mec.web.dal.enums.ClassGroupTypeEnum;
import com.ym.mec.web.dal.enums.TeachTypeEnum;
import com.ym.mec.web.dal.page.VipClassQueryInfo;
import com.ym.mec.web.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.util.*;

@Service
public class ClassGroupServiceImpl extends BaseServiceImpl<Integer, ClassGroup> implements ClassGroupService {

    @Autowired
    private ClassGroupDao classGroupDao;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private ClassGroupRelationService classGroupRelationService;
    @Autowired
    private ClassGroupStudentMapperService classGroupStudentMapperService;
    @Autowired
    private ClassGroupTeacherMapperService classGroupTeacherMapperService;
    @Autowired
    private TeacherDefaultMusicGroupSalaryService teacherDefaultMusicGroupSalaryService;
    @Autowired
    private MusicGroupService musicGroupService;

    @Override
    public BaseDAO<Integer, ClassGroup> getDAO() {
        return classGroupDao;
    }

    @Override
    public List<ClassGroup> findClassGroup4Teacher(Integer teacherId) {
        return classGroupDao.findClassGroup4Teacher(teacherId);
    }

    @Override
    public List<ClassGroup> findClassGroup(Integer musicGroupId, Integer mixClassGroupId) {
        if (null == mixClassGroupId) {
            return classGroupDao.findAllMixClassGroup(musicGroupId);
        } else {
            return classGroupDao.findMixClassChildClassGroup(mixClassGroupId);
        }
    }

    @Override
    public List<ClassGroup> findAllNormalClassGroupByMusicGroupId(int musicGroupId) {
        return classGroupDao.findAllNormalClassGroupByMusicGroupId(musicGroupId);
    }

    @Override
    public List<ClassGroup> findAllMixClassGroupByMusicGroupId(int musicGroupId) {
        List<ClassGroup> allMixClassGroup = classGroupDao.findAllMixClassGroup(musicGroupId);
        for (ClassGroup mixClassGroup : allMixClassGroup) {
            mixClassGroup.setClassNames(classGroupDao.findMixClassChildClassGroupNames(mixClassGroup.getId()).getName());
        }
        return allMixClassGroup;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ClassGroup addClassGroup(ClassGroup classGroup) throws Exception {
        Date date;
        date = new Date();
        classGroup.setCreateTime(date);
        classGroup.setUpdateTime(date);
        classGroup.setType(ClassGroupTypeEnum.NORMAL);
        String userIds = classGroup.getUserIds();
        String[] userIdArr = userIds.split(",");

        classGroup.setStudentNum(userIdArr.length);
        Long classGroupId = this.insert(classGroup);

        //2、插入班级学生关联关系
        List<ClassGroupStudentMapper> classGroupStudentList = new ArrayList<>();
        for (String userId : userIdArr) {
            ClassGroupStudentMapper classGroupStudentMapper = new ClassGroupStudentMapper();
            classGroupStudentMapper.setClassGroupId(classGroupId.intValue());
            classGroupStudentMapper.setUserId(Integer.getInteger(userId));
            classGroupStudentMapper.setCreateTime(date);
            classGroupStudentMapper.setStatus(ClassGroupStudentStatusEnum.NORMAL);
            classGroupStudentList.add(classGroupStudentMapper);
        }
        classGroupStudentMapperService.classGroupStudentsInsert(classGroupStudentList);

        return classGroup;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ClassGroup addMixClassGroup(int musicGroupId, String name, String classGroupIds) throws Exception {
        Date date;
        date = new Date();

        List<ClassGroup> classGroups = findClassGroupByIds(classGroupIds);
        int studentNum = 0; //学生数
        String subjectIds = "";
        for (ClassGroup classGroup : classGroups) {
            studentNum += classGroup.getStudentNum();
            subjectIds += subjectIds.isEmpty() ? classGroup.getSubjectIdList() : "," + classGroup.getSubjectIdList();
        }

        //1、插入班级
        ClassGroup classGroup = new ClassGroup();
        classGroup.setMusicGroupId(musicGroupId);
        classGroup.setSubjectIdList(subjectIds);
        classGroup.setName(name);
        classGroup.setType(ClassGroupTypeEnum.MIX);
        classGroup.setCreateTime(date);
        classGroup.setUpdateTime(date);
        Long classGroupId = this.insert(classGroup);

        //2、插入班级关联关系
        List<ClassGroupRelation> classGroupRelationList = new ArrayList<>();
        for (ClassGroup cGroup : classGroups) {
            ClassGroupRelation classGroupRelation = new ClassGroupRelation();
            classGroupRelation.setClassGroupId(classGroupId.intValue());
            classGroupRelation.setSubClassGroupId(cGroup.getId());
            classGroupRelation.setCreateTime(date);
            classGroupRelationList.add(classGroupRelation);
        }
        classGroupRelationService.classGroupRelationsInsert(classGroupRelationList);
        return classGroup;
    }

    @Override
    public List<ClassGroup> findClassGroupByIds(String ids) {
        return classGroupDao.findClassGroupByIds(ids);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<HighClassGroup> addHighClassGroup(List<HighClassGroup> highClassGroupList) throws Exception {
        LocalDate now = LocalDate.now();
        for (HighClassGroup highClassGroup : highClassGroupList) {
            //1、插入班级信息
            Date date;
            date = new Date();
            highClassGroup.setCreateTime(date);
            highClassGroup.setUpdateTime(date);
            highClassGroup.setType(ClassGroupTypeEnum.HIGH);
            Long classGroupId = this.insert(highClassGroup);

            //2、插入班级关联老师
            TeacherDefaultMusicGroupSalary teacherSalary = teacherDefaultMusicGroupSalaryService.getTeacherSalaryByUserIdAndType(highClassGroup.getUserId(), ClassGroupTypeEnum.HIGH);
            ClassGroupTeacherMapper classGroupTeacherMapper = new ClassGroupTeacherMapper();
            classGroupTeacherMapper.setMusicGroupId(highClassGroup.getUserId());
            classGroupTeacherMapper.setClassGroupId(classGroupId.intValue());
            classGroupTeacherMapper.setTeacherRole(TeachTypeEnum.BISHOP);
            classGroupTeacherMapper.setUserId(highClassGroup.getUserId());
            classGroupTeacherMapper.setSalary(teacherSalary.getMainTeacherSalary());
            classGroupTeacherMapper.setCreateTime(date);
            classGroupTeacherMapper.setUpdateTime(date);
            classGroupTeacherMapperService.insert(classGroupTeacherMapper);

            //3、插入班级排课信息
            int i = 0;
            while (true) {
                int dayOfWeek = now.plusDays(1).getDayOfWeek().getValue();
                if (highClassGroup.getDayOfWeek().equals(dayOfWeek)) {

                }
            }


        }
        return highClassGroupList;
    }

    @Override
    public List<TeacherMusicClassInfoDto> getTeacherMusicClass(Integer teacherId) {
        List<TeacherMusicClassInfoDto> teacherMusicClassInfoDtos = classGroupDao.queryGroupCourses(teacherId);
        Teacher teacher = teacherDao.get(teacherId);
        if(teacher == null){
            return null;
        }
        teacherMusicClassInfoDtos.forEach(e->{
            e.setJobNature(teacher.getJobNature());
        });
        return teacherMusicClassInfoDtos;
    }

    @Override
    public PageInfo<TeacherVipClassInfoDto> getTeacherVipClass(VipClassQueryInfo queryInfo) {
        PageInfo<TeacherVipClassInfoDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);
        List<TeacherVipClassInfoDto> dataList = null;
        int count = classGroupDao.countTeacherVipClass(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = classGroupDao.getTeacherVipClass(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }
}